<#ibiztemplate>
TARGET=PSSYSTEM
</#ibiztemplate>
<#if sys.codeName == 'Sample'>
package ${pub.getPKGCodeName()}.util.service;

import ${pub.getPKGCodeName()}.util.annotation.ValueRuleCheck;
import ${pub.getPKGCodeName()}.util.domain.DTOBase;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.util.ObjectUtils;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.util.HashMap;
import java.util.Map;

public class ValueRuleValidator implements ConstraintValidator<ValueRuleCheck, DTOBase> {

    private static Map<String, String> sysRule = new HashMap<>();

    private final ExpressionParser parser = new SpelExpressionParser();

    private String field;

    private String rule;

    @Override
    public void initialize(ValueRuleCheck constraintAnnotation) {
        this.field = constraintAnnotation.field();
        this.rule = constraintAnnotation.rule();
    }

    @Override
    public boolean isValid(DTOBase dto, ConstraintValidatorContext constraintContext) {
        if (ObjectUtils.isEmpty(dto) || ObjectUtils.isEmpty(rule) || ObjectUtils.isEmpty(sysRule.get(rule))) {
            return true;
        }
        EvaluationContext searchContextCtx = new StandardEvaluationContext();
        searchContextCtx.setVariable("dto", dto);
        Expression searchContextExp = parser.parseExpression(sysRule.get(rule));
        return searchContextExp.getValue(searchContextCtx, Boolean.class);
    }

    static {
        <#if sys.getAllPSDataEntities()??>
            <#list sys.getAllPSDataEntities() as de>
                <#if de.getAllPSDEFValueRules()??>
                <#list de.getAllPSDEFValueRules() as deRule>
                    <#if deRule.codeName!='Default'>
                        <#if deRule.getPSDEFVRGroupCondition()??>
                            <#assign group=deRule.getPSDEFVRGroupCondition()>
                            <#assign ruleName=de.codeName+"_"+deRule.getPSDEField().codeName+"_"+deRule.codeName>
                            <#assign ruleCond=getGroupCond(group)>
        sysRule.put("${ruleName}", "${ruleCond}");
                        </#if>
                    </#if>
                </#list>
                </#if>
            </#list>
        </#if>

    }

}
</#if>

<#comment>获取组条件表达式</#comment>
<#function getGroupCond group>
    <#assign strRuleCond="(">
    <#if group.getPSDEFVRConditions()??>
        <#assign condOp=group.getCondOp()?replace("AND","&&")?replace("OR","||")>
        <#list group.getPSDEFVRConditions() as childRule>
            <#assign childRuleType=childRule.getCondType()>
            <#if childRuleType=='GROUP'>
                <#assign strRuleCond=strRuleCond+getGroupCond(childRule)>
            <#else>
                <#assign strRuleCond=strRuleCond+getFieldCond(childRule)>
            </#if>
            <#if childRule_has_next>
                <#assign strRuleCond=strRuleCond+condOp>//拼接连接符
            </#if>
        </#list>
    </#if>
    <#assign strRuleCond=strRuleCond+")">
    <#return strRuleCond/>
</#function>

<#comment>获取单项条件表达式</#comment>
<#function getFieldCond fieldRule>
    <#assign fieldCond="(" >
    <#assign condBody="">
    <#assign targetFieldName=fieldRule.getDEFName()><#comment>目标属性名称</#comment>
    <#assign targetDEField=((de.getPSDEField(targetFieldName,true))!'')><#comment>目标属性</#comment>
    <#assign targetDBValueOP=fieldRule.getCondOp()><#comment>表达式</#comment>
    <#assign targetValue=((fieldRule.getParamValue())!'')><#comment>值项</#comment>
    <#if targetDEField!=''>
        <#assign targetFieldName=srfcaseformat(targetDEField.codeName,'l_u2lC')>
    </#if>
    <#assign condBody="T(${pub.getPKGCodeName()}.util.helper.RuleUtils).test(#dto.get(\""+targetFieldName+"\"),\""+targetDBValueOP+"\","+"\""+targetValue+"\")">
    <#assign fieldCond=fieldCond+condBody >
    <#assign fieldCond=fieldCond+")" >
    <#return fieldCond/>
</#function>